@@ -26,54 +26,32 @@
 #endif
 #include "pixman-private.h"
 
-static void
-solid_fill_get_scanline_32 (pixman_image_t *image,
-                            int             x,
-                            int             y,
-                            int             width,
-                            uint32_t *      buffer,
-                            const uint32_t *mask)
+void
+_pixman_solid_fill_iter_init (pixman_image_t *image,
+			      pixman_iter_t  *iter,
+			      int x, int y, int width, int height,
+			      uint8_t *buffer, iter_flags_t flags)
 {
-    uint32_t *end = buffer + width;
-    uint32_t color = image->solid.color_32;
+    if (flags & ITER_NARROW)
+    {
+	uint32_t *b = (uint32_t *)buffer;
+	uint32_t *e = b + width;
+	uint32_t color = image->solid.color_32;
 
-    while (buffer < end)
-	*(buffer++) = color;
+	while (b < e)
+	    *(b++) = color;
+    }
+    else
+    {
+	uint64_t *b = (uint64_t *)buffer;
+	uint64_t *e = b + width;
+	uint64_t color = image->solid.color_64;
 
-    return;
-}
+	while (b < e)
+	    *(b++) = color;
+    }
 
-static void
-solid_fill_get_scanline_64 (pixman_image_t *image,
-			    int             x,
-			    int             y,
-			    int             width,
-			    uint32_t *      buffer,
-			    const uint32_t *mask)
-{
-    uint64_t *b = (uint64_t *)buffer;
-    uint64_t *e = b + width;
-    uint64_t color = image->solid.color_64;
-
-    while (b < e)
-	*(b++) = color;
-}
-
-static source_image_class_t
-solid_fill_classify (pixman_image_t *image,
-                     int             x,
-                     int             y,
-                     int             width,
-                     int             height)
-{
-    return SOURCE_IMAGE_CLASS_HORIZONTAL;
-}
-
-static void
-solid_fill_property_changed (pixman_image_t *image)
-{
-    image->common.get_scanline_32 = solid_fill_get_scanline_32;
-    image->common.get_scanline_64 = solid_fill_get_scanline_64;
+    iter->get_scanline = _pixman_iter_get_scanline_noop;
 }
 
 static uint32_t
@@ -109,9 +87,6 @@ pixman_image_create_solid_fill (pixman_color_t *color)
     img->solid.color_32 = color_to_uint32 (color);
     img->solid.color_64 = color_to_uint64 (color);
 
-    img->common.classify = solid_fill_classify;
-    img->common.property_changed = solid_fill_property_changed;
-
     return img;
 }
 
